#! /usr/bin/env python
from version import __appname__, __version__  # NOQA
import datetime
import os
import sys

from scrib import barf, cfg, logger, process


if sys.version_info[0] >= 3:
	xrange = range
	unicode = str
	basestring = str


def name(name):
	"""
	Simple wrapper for cfg.get_name()
	"""
	try:
		with open(os.path.expanduser('~') + '/.scrib/current', 'r') as f:
			name = f.readlines()
		if len(name):
			name = name[0]
		return name
	except IOError:
		if name is '':
			name = 'scrib'  # set default
		cfg.get_name(name)
		return name


class scrib:
	"""
	Set us up the scrib.
	"""
	def __init__(self, interface, args):
		"""
		Here we'll load settings and set up modules.
		"""
		# Expose these for others to tap into
		self.barf = barf.Barf
		self.process = process.Process

		# Load scrib settings
		self.settings = self.setcfg()
		self.settings.load('scrib.cfg', {}, suppress=True)
		self.version = __version__

		# Load interface settings
		self.interface = self.setcfg()
		self.name = name('')
		args['name'] = __name__
		self.interface.load(interface + '.cfg', args, suppress=True)

		initmsg = 'scrib v%s started' % self.version
		self.log(interface, self.name, initmsg)
		
		if self.settings.debug is 1:
			self.barf('DBG', initmsg)

	def setcfg(self):
		"""
		Simple wrapper for cfg.set()
		"""
		return cfg.set()

	def save(self):
		self.barf('SAV', 'Saving settings')
		self.settings.save()

	def shutdown(self, interface=False):
		"""
		Shuts us the scrib down.
		"""
		msg = 'Shutting down. Goodbye!'
		try:
			if 'quitmsg' in self.interface._defaults:
				msg = self.interface._defaults['quitmsg']
		except:
			pass

		try:
			if interface is not False:
				self.log(interface, self.name, msg)
				self.reply(interface, self.name, msg, reply='', bypass=True)
		except AttributeError as e:
			self.barf('ERR', 'scrib.shutdown: %s' % e)
			sys.exit()

		self.save()
		self.barf('MSG', msg)
		sys.exit()

	def log(self, interface, sender, message):
		timestamp = str(datetime.time(1, 2, 3))
		logme = "[%s] %s <%s> %s" % (timestamp, interface, sender, message)
		logger.log(logme)

	def reply(self, interface, name, message, reply='', learn=True, bypass=False):
		"""
		Send reply, or bypass if told to.
		"""
		if bypass is True:
			return message

		if message[1:] == '!':
			if self.settings.debug is 1:
				self.barf('DBG', 'scrib.reply: Abort; is command.')
			return

		if '://' in message:
			if self.settings.debug is 1:
				self.barf('DBG', 'scrib.reply: Abort; link detected.')
			return

		try:
			messages = self.process(interface).msg(name, message, reply, learn)
			return messages

		except UnicodeDecodeError as e:
			# Don't need to crash every goddamn time, yo.
			self.barf('ERR', 'scrib.reply: %s' % e)
			self.barf('ACT', 'Ignoring...')
			# Try again
			if self.settings.debug is 1:
				self.barf('DBG', 'Retrying last message...')
			self.reply(interface, name, message, reply, learn, bypass)
