#! /usr/bin/env python
from datetime import datetime

from scrib import barf, cfg, clean, db


class Process:
	"""
	Setting up processing.
	"""
	def __init__(self, interface):
		self.settings = cfg.set()
		# Don't set defaults here.
		self.settings.load('scrib.cfg', {}, suppress=True)
		self.interface = cfg.set()
		# Don't set defaults here.
		self.interface.load(interface + '.cfg', {}, suppress=True)
		self.barf = barf.Barf
		self.clean = clean.Clean(self.settings.debug)
		self.messages_to_generate = 1  # In case you want to allow multiple responses
		self.separator = ' '

	def msg(self, name, data, learn, reply=''):
		messages = []
		if self.settings.debug is 1:
			self.barf('DBG', 'process.msg: %s' % data)

		sentences = self.split_to_sentences(data)
		
		for sentence in sentences:
			s = db.Sentence()
			s.src = name
			s.src_input = sentence
			s.sentiment = self.sentiment(s.src_input)
			s.timestamp = datetime.utcnow()
			s.insert()

			# Don't want to do this yet.
			"""
			sentence = self.clean.split(self.clean.sanitize(sentence))
			for words in sentence:
				for word in words:
					if self.settings.debug is 1:
						self.barf('DBG', 'process.msg: "%s" saved' % word)
					w = db.Word()
					w.word = word
					w.insert()
			"""

		if len(messages) and reply:
			if self.settings.debug is 1:
				self.barf('DBG', 'process.msg: replying')
			return max(messages, key=len)

		if self.settings.debug is 1:
				self.barf('DBG', 'process.msg: not replying')

	def split_to_sentences(self, message):
		message = str(message)
		try:
			try:
				message = [x for x in map(str.strip, message.split('.')) if x]
			except AttributeError as e:
				self.barf('ERR', 'process.split_to_sentences: AttributeError: %s' % e)
				
			return message
		except Exception as e:
			self.barf('ERR', 'process.split_to_sentences: %s' % e)

	def sentiment(self, sentence):
		return 0  # this sets it to neutral, for now.

	def generate(self, seed):
		"""
		GENERATE ALL THE THINGS
		"""
		return ''

		"""
		if self.settings.debug is 1:
			self.barf('DBG', 'process.generate: Seed: %s ' % seed)
		key = seed
		gen_words = []

		for i in xrange(self.interface.maxwords):
			words = key.split(self.separator)
			gen_words.append(words[0])
			next_word = self.db.get_random(key)
			if self.settings.debug is 1:
				self.barf('DBG', 'process.generate: Next word: %s' % next_word)
			if not next_word:
				break

			key = self.separator.join(words[1:] + [next_word])

		return ' '.join(gen_words)
		"""
